@@ -73,20 +73,16 @@ assets.fetch = function(url, options = {}) {
     return new Promise((resolve, reject) => {
     // Start of executor
 
-    const timeoutAfter = µb.hiddenSettings.assetFetchTimeout * 1000 || 30000;
+    const timeoutAfter = µb.hiddenSettings.assetFetchTimeout || 30;
     const xhr = new XMLHttpRequest();
     let contentLoaded = 0;
-    let timeoutTimer;
 
     const cleanup = function() {
         xhr.removeEventListener('load', onLoadEvent);
         xhr.removeEventListener('error', onErrorEvent);
         xhr.removeEventListener('abort', onErrorEvent);
         xhr.removeEventListener('progress', onProgressEvent);
-        if ( timeoutTimer !== undefined ) {
-            clearTimeout(timeoutTimer);
-            timeoutTimer = undefined;
-        }
+        timeoutTimer.off();
     };
 
     const fail = function(details, msg) {
@@ -130,12 +126,11 @@ assets.fetch = function(url, options = {}) {
     const onProgressEvent = function(ev) {
         if ( ev.loaded === contentLoaded ) { return; }
         contentLoaded = ev.loaded;
-        if ( timeoutTimer !== undefined ) {
-            clearTimeout(timeoutTimer); 
-        }
-        timeoutTimer = vAPI.setTimeout(onTimeout, timeoutAfter);
+        timeoutTimer.offon({ sec: timeoutAfter });
     };
 
+    const timeoutTimer = vAPI.defer.create(onTimeout);
+
     // Be ready for thrown exceptions:
     // I am pretty sure it used to work, but now using a URL such as
     // `file:///` on Chromium 40 results in an exception being thrown.
@@ -147,7 +142,7 @@ assets.fetch = function(url, options = {}) {
         xhr.addEventListener('progress', onProgressEvent);
         xhr.responseType = options.responseType || 'text';
         xhr.send();
-        timeoutTimer = vAPI.setTimeout(onTimeout, timeoutAfter);
+        timeoutTimer.on({ sec: timeoutAfter });
     } catch (e) {
         onErrorEvent.call(xhr);
     }
@@ -202,6 +197,7 @@ assets.fetchText = async function(url) {
         const text = details.content.trim();
         if ( text.startsWith('<') && text.endsWith('>') ) {
             details.content = '';
+            details.error = 'assets.fetchText(): Not a text file';
         }
 
         // Important: Non empty text resource must always end with a newline
@@ -364,11 +360,11 @@ const getAssetSourceRegistry = function() {
                 return assetSourceRegistry;
             }
             return assets.fetchText(
-                µb.assetsBootstrapLocation || 'assets/assets.json'
+                µb.assetsBootstrapLocation || µb.assetsJsonPath
             ).then(details => {
                 return details.content !== ''
                     ? details
-                    : assets.fetchText('assets/assets.json');
+                    : assets.fetchText(µb.assetsJsonPath);
             }).then(details => {
                 updateAssetSourceRegistry(details.content, true);
                 return assetSourceRegistry;
@@ -379,22 +375,21 @@ const getAssetSourceRegistry = function() {
     return assetSourceRegistryPromise;
 };
 
-const registerAssetSource = function(assetKey, dict) {
-    const entry = assetSourceRegistry[assetKey] || {};
-    for ( const prop in dict ) {
-        if ( dict.hasOwnProperty(prop) === false ) { continue; }
-        if ( dict[prop] === undefined ) {
-            delete entry[prop];
+const registerAssetSource = function(assetKey, newDict) {
+    const currentDict = assetSourceRegistry[assetKey] || {};
+    for ( const [ k, v ] of Object.entries(newDict) ) {
+        if ( v === undefined || v === null ) {
+            delete currentDict[k];
         } else {
-            entry[prop] = dict[prop];
+            currentDict[k] = newDict[k];
         }
     }
-    let contentURL = dict.contentURL;
+    let contentURL = newDict.contentURL;
     if ( contentURL !== undefined ) {
         if ( typeof contentURL === 'string' ) {
-            contentURL = entry.contentURL = [ contentURL ];
+            contentURL = currentDict.contentURL = [ contentURL ];
         } else if ( Array.isArray(contentURL) === false ) {
-            contentURL = entry.contentURL = [];
+            contentURL = currentDict.contentURL = [];
         }
         let remoteURLCount = 0;
         for ( let i = 0; i < contentURL.length; i++ ) {
@@ -402,18 +397,18 @@ const registerAssetSource = function(assetKey, dict) {
                 remoteURLCount += 1;
             }
         }
-        entry.hasLocalURL = remoteURLCount !== contentURL.length;
-        entry.hasRemoteURL = remoteURLCount !== 0;
-    } else if ( entry.contentURL === undefined ) {
-        entry.contentURL = [];
+        currentDict.hasLocalURL = remoteURLCount !== contentURL.length;
+        currentDict.hasRemoteURL = remoteURLCount !== 0;
+    } else if ( currentDict.contentURL === undefined ) {
+        currentDict.contentURL = [];
     }
-    if ( typeof entry.updateAfter !== 'number' ) {
-        entry.updateAfter = 5;
+    if ( typeof currentDict.updateAfter !== 'number' ) {
+        currentDict.updateAfter = 7;
     }
-    if ( entry.submitter ) {
-        entry.submitTime = Date.now(); // To detect stale entries
+    if ( currentDict.submitter ) {
+        currentDict.submitTime = Date.now(); // To detect stale entries
     }
-    assetSourceRegistry[assetKey] = entry;
+    assetSourceRegistry[assetKey] = currentDict;
 };
 
 const unregisterAssetSource = function(assetKey) {
@@ -422,17 +417,14 @@ const unregisterAssetSource = function(assetKey) {
 };
 
 const saveAssetSourceRegistry = (( ) => {
-    let timer;
-    const save = function() {
-        timer = undefined;
+    const save = ( ) => {
+        timer.off();
         cacheStorage.set({ assetSourceRegistry });
     };
+    const timer = vAPI.defer.create(save);
     return function(lazily) {
-        if ( timer !== undefined ) {
-            clearTimeout(timer);
-        }
         if ( lazily ) {
-            timer = vAPI.setTimeout(save, 500);
+            timer.offon(500);
         } else {
             save();
         }
@@ -443,12 +435,18 @@ const updateAssetSourceRegistry = function(json, silent = false) {
     let newDict;
     try {
         newDict = JSON.parse(json);
+        newDict['assets.json'].defaultListset =
+            Array.from(Object.entries(newDict))
+                .filter(a => a[1].content === 'filters' && a[1].off === undefined)
+                .map(a => a[0]);
     } catch (ex) {
     }
     if ( newDict instanceof Object === false ) { return; }
 
     const oldDict = assetSourceRegistry;
 
+    fireNotification('assets.json-updated', { newDict, oldDict });
+
     // Remove obsolete entries (only those which were built-in).
     for ( const assetKey in oldDict ) {
         if (
@@ -527,15 +525,14 @@ const getAssetCacheRegistry = function() {
 };
 
 const saveAssetCacheRegistry = (( ) => {
-    let timer;
     const save = function() {
-        timer = undefined;
+        timer.off();
         cacheStorage.set({ assetCacheRegistry });
     };
+    const timer = vAPI.defer.create(save);
     return function(lazily) {
-        if ( timer !== undefined ) { clearTimeout(timer); }
         if ( lazily ) {
-            timer = vAPI.setTimeout(save, 30000);
+            timer.offon({ sec: 30 });
         } else {
             save();
         }
@@ -558,10 +555,12 @@ const assetCacheRead = async function(assetKey, updateReadTime = false) {
         cacheStorage.get(internalKey),
     ]);
 
-    µb.supportStats.maxAssetCacheWait = Math.max(
-        Date.now() - t0,
-        parseInt(µb.supportStats.maxAssetCacheWait, 10)
-    ) + ' ms';
+    if ( µb.readyToFilter !== true ) {
+        µb.supportStats.maxAssetCacheWait = Math.max(
+            Date.now() - t0,
+            parseInt(µb.supportStats.maxAssetCacheWait, 10) || 0
+        ) + ' ms';
+    }
 
     if (
         bin instanceof Object === false ||
@@ -779,6 +778,7 @@ assets.get = async function(assetKey, options = {}) {
         contentURLs.push(...assetDetails.cdnURLs);
     }
 
+    let error = 'ENOTFOUND';
     for ( const contentURL of contentURLs ) {
         if ( reIsExternalPath.test(contentURL) && assetDetails.hasLocalURL ) {
             continue;
@@ -786,6 +786,9 @@ assets.get = async function(assetKey, options = {}) {
         const details = assetDetails.content === 'filters'
             ? await assets.fetchFilterList(contentURL)
             : await assets.fetchText(contentURL);
+        if ( details.error !== undefined ) {
+            error = details.error;
+        }
         if ( details.content === '' ) { continue; }
         if ( reIsExternalPath.test(contentURL) && options.dontCache !== true ) {
             assetCacheWrite(assetKey, {
@@ -793,10 +796,16 @@ assets.get = async function(assetKey, options = {}) {
                 url: contentURL,
                 silent: options.silent === true,
             });
+            registerAssetSource(assetKey, { error: undefined });
         }
         return reportBack(details.content, contentURL);
     }
-    return reportBack('', '', 'ENOTFOUND');
+    if ( assetRegistry[assetKey] !== undefined ) {
+        registerAssetSource(assetKey, {
+            error: { time: Date.now(), error }
+        });
+    }
+    return reportBack('', '', error);
 };
 
 /******************************************************************************/
@@ -806,7 +815,7 @@ const getRemote = async function(assetKey) {
     const assetDetails = assetRegistry[assetKey] || {};
 
     const reportBack = function(content, err) {
-        const details = { assetKey: assetKey, content: content };
+        const details = { assetKey, content };
         if ( err ) {
             details.error = assetDetails.lastError = err;
         } else {
@@ -842,9 +851,19 @@ const getRemote = async function(assetKey) {
         }
     }
 
-    for ( const contentURL of contentURLs ) {
+    for ( let contentURL of contentURLs ) {
         if ( reIsExternalPath.test(contentURL) === false ) { continue; }
 
+        // This will force uBO to fetch the proper version according to whether
+        // the dev build is being used. This can be removed when execution of
+        // this code path is widespread for dev build revisions of uBO.
+        if ( assetKey === 'assets.json' ) {
+            contentURL = contentURL.replace(
+                /\/assets\/assets\.json$/,
+                µb.assetsJsonPath
+            );
+        }
+
         const result = assetDetails.content === 'filters'
             ? await assets.fetchFilterList(contentURL)
             : await assets.fetchText(contentURL);
@@ -937,13 +956,35 @@ assets.rmrf = function() {
 
 /******************************************************************************/
 
+assets.getUpdateAges = async function(conditions = {}) {
+    const assetDict = await assets.metadata();
+    const now = Date.now();
+    const out = [];
+    for ( const [ assetKey, asset ] of Object.entries(assetDict) ) {
+        if ( asset.hasRemoteURL !== true ) { continue; }
+        const tokens = conditions[asset.content];
+        if ( Array.isArray(tokens) === false ) { continue; }
+        if ( tokens.includes('*') === false ) {
+            if ( tokens.includes(assetKey) === false ) { continue; }
+        }
+        const age = now  - (asset.writeTime || 0);
+        out.push({
+            assetKey,
+            age,
+            ageNormalized: age / (asset.updateAfter * 86400000),
+        });
+    }
+    return out;
+};
+
+/******************************************************************************/
+
 // Asset updater area.
 const updaterAssetDelayDefault = 120000;
 const updaterUpdated = [];
 const updaterFetched = new Set();
 
 let updaterStatus;
-let updaterTimer;
 let updaterAssetDelay = updaterAssetDelayDefault;
 let updaterAuto = false;
 
@@ -1003,7 +1044,16 @@ const updateNext = async function() {
     // In auto-update context, be gentle on remote servers.
     remoteServerFriendly = updaterAuto;
 
-    const result = await getRemote(toUpdate[0]);
+    let result;
+    if (
+        toUpdate[0] !== 'assets.json' ||
+        µb.hiddenSettings.debugAssetsJson !== true
+    ) {
+        result = await getRemote(toUpdate[0]);
+    } else {
+        result = await assets.fetchText(µb.assetsJsonPath);
+        result.assetKey = 'assets.json';
+    }
 
     remoteServerFriendly = false;
 
@@ -1016,9 +1066,11 @@ const updateNext = async function() {
         fireNotification('asset-update-failed', { assetKey: result.assetKey });
     }
 
-    vAPI.setTimeout(updateNext, updaterAssetDelay);
+    updaterTimer.on(updaterAssetDelay);
 };
 
+const updaterTimer = vAPI.defer.create(updateNext);
+
 const updateDone = function() {
     const assetKeys = updaterUpdated.slice(0);
     updaterFetched.clear();
@@ -1037,8 +1089,7 @@ assets.updateStart = function(details) {
     updaterAuto = details.auto === true;
     if ( updaterStatus !== undefined ) {
         if ( newUpdateDelay < oldUpdateDelay ) {
-            clearTimeout(updaterTimer);
-            updaterTimer = vAPI.setTimeout(updateNext, updaterAssetDelay);
+            updaterTimer.offon(updaterAssetDelay);
         }
         return;
     }
@@ -1046,10 +1097,7 @@ assets.updateStart = function(details) {
 };
 
 assets.updateStop = function() {
-    if ( updaterTimer ) {
-        clearTimeout(updaterTimer);
-        updaterTimer = undefined;
-    }
+    updaterTimer.off();
     if ( updaterStatus !== undefined ) {
         updateDone();
     }
