@@ -26,6 +26,7 @@
 /******************************************************************************/
 
 import * as sfp from '../static-filtering-parser.js';
+import { dom, qs$ } from '../dom.js';
 
 /******************************************************************************/
 
@@ -36,10 +37,19 @@ const preparseDirectiveHints = [];
 const originHints = [];
 let hintHelperRegistered = false;
 
+
+/******************************************************************************/
+
+let filterOnHeaders = false;
+
+CodeMirror.defineOption('filterOnHeaders', false, (cm, state) => {
+    filterOnHeaders = state;
+    self.dispatchEvent(new Event('filterOnHeaders'));
+});
+
 /******************************************************************************/
 
 CodeMirror.defineMode('ubo-static-filtering', function() {
-    if ( sfp.AstFilterParser instanceof Object === false ) { return; }
     const astParser = new sfp.AstFilterParser({
         interactive: true,
         nativeCssHas: vAPI.webextFlavor.env.includes('native_css_has'),
@@ -80,9 +90,6 @@ CodeMirror.defineMode('ubo-static-filtering', function() {
                 const raw = astParser.getNodeString(currentWalkerNode);
                 const not = raw.startsWith('!');
                 const token = not ? raw.slice(1) : raw;
-                if ( preparseDirectiveTokens.has(token) === false ) {
-                    return 'error strong';
-                }
                 return not === preparseDirectiveTokens.get(token)
                     ? 'negative strong'
                     : 'positive strong';
@@ -209,6 +216,10 @@ CodeMirror.defineMode('ubo-static-filtering', function() {
         return '+';
     };
 
+    self.addEventListener('filterOnHeaders', ( ) => {
+        astParser.options.filterOnHeaders = filterOnHeaders;
+    });
+
     return {
         lineComment: '!',
         token: function(stream) {
@@ -299,8 +310,6 @@ CodeMirror.defineMode('ubo-static-filtering', function() {
 //   https://codemirror.net/demo/complete.html
 
 const initHints = function() {
-    if ( sfp.AstFilterParser instanceof Object === false ) { return; }
-
     const astParser = new sfp.AstFilterParser({
         interactive: true,
         nativeCssHas: vAPI.webextFlavor.env.includes('native_css_has'),
@@ -665,6 +674,348 @@ CodeMirror.registerHelper('fold', 'ubo-static-filtering', (( ) => {
 
 /******************************************************************************/
 
+// Linter
+
+{
+    const astParser = new sfp.AstFilterParser({
+        interactive: true,
+        nativeCssHas: vAPI.webextFlavor.env.includes('native_css_has'),
+    });
+
+    const changeset = [];
+    let changesetTimer;
+
+    const includeset = new Set();
+    let errorCount = 0;
+
+    const extractMarkerDetails = (doc, lineHandle) => {
+        if ( astParser.isUnsupported() ) {
+            return { value: 'error', msg: 'Unsupported filter syntax' };
+        }
+        if ( astParser.hasError() ) {
+            let msg = 'Invalid filter';
+            switch ( astParser.astError ) {
+                case sfp.AST_ERROR_UNSUPPORTED:
+                    msg = `${msg}: Unsupported filter syntax`;
+                    break;
+                case sfp.AST_ERROR_REGEX:
+                    msg = `${msg}: Bad regular expression`;
+                    break;
+                case sfp.AST_ERROR_PATTERN:
+                    msg = `${msg}: Bad pattern`;
+                    break;
+                case sfp.AST_ERROR_DOMAIN_NAME:
+                    msg = `${msg}: Bad domain name`;
+                    break;
+                case sfp.AST_ERROR_OPTION_DUPLICATE:
+                    msg = `${msg}: Duplicate filter option`;
+                    break;
+                case sfp.AST_ERROR_OPTION_UNKNOWN:
+                    msg = `${msg}: Unsupported filter option`;
+                    break;
+                case sfp.AST_ERROR_IF_TOKEN_UNKNOWN:
+                    msg = `${msg}: Unknown preparsing token`;
+                    break;
+                default:
+                    if ( astParser.isCosmeticFilter() && astParser.result.error ) {
+                        msg = `${msg}: ${astParser.result.error}`;
+                    }
+                    break;
+            }
+            return { value: 'error', msg };
+        }
+        if ( astParser.astType !== sfp.AST_TYPE_COMMENT ) { return; }
+        if ( astParser.astTypeFlavor !== sfp.AST_TYPE_COMMENT_PREPARSER ) {
+            if ( astParser.raw.startsWith('! <<<<<<<< ') === false ) { return; }
+            for ( const include of includeset ) {
+                if ( astParser.raw.endsWith(include) === false ) { continue; }
+                includeset.delete(include);
+                return { value: 'include-end' };
+            }
+            return;
+        }
+        if ( /^\s*!#if \S+/.test(astParser.raw) ) {
+            return { value: 'if-start' };
+        }
+        if ( /^\s*!#endif\b/.test(astParser.raw) ) {
+            return { value: 'if-end' };
+        }
+        const match = /^\s*!#include\s*(\S+)/.exec(astParser.raw);
+        if ( match === null ) { return; }
+        const nextLineHandle = doc.getLineHandle(lineHandle.lineNo() + 1);
+        if ( nextLineHandle === undefined ) { return; }
+        if ( nextLineHandle.text.startsWith('! >>>>>>>> ') === false ) { return; }
+        const includeToken = `/${match[1]}`;
+        if ( nextLineHandle.text.endsWith(includeToken) === false ) { return; }
+        includeset.add(includeToken);
+        return { value: 'include-start' };
+    };
+
+    const extractMarker = lineHandle => {
+        const markers = lineHandle.gutterMarkers || null;
+        return markers !== null
+            ? markers['CodeMirror-lintgutter'] || null
+            : null;
+    };
+
+    const markerTemplates = {
+        'error': {
+            node: null,
+            html: [
+                '<div class="CodeMirror-lintmarker" data-lint="error">&nbsp;',
+                  '<span class="msg"></span>',
+                '</div>',
+            ],
+        },
+        'if-start': {
+            node: null,
+            html: [
+                '<div class="CodeMirror-lintmarker" data-lint="if" data-fold="start">&nbsp;',
+                  '<svg viewBox="0 0 100 100">',
+                    '<polygon points="0,0 100,0 50,100" />',
+                  '</svg>',
+                '</div>',
+            ],
+        },
+        'if-end': {
+            node: null,
+            html: [
+                '<div class="CodeMirror-lintmarker" data-lint="if" data-fold="end">&nbsp;',
+                  '<svg viewBox="0 0 100 100">',
+                    '<polygon points="50,0 100,100 0,100" />',
+                  '</svg>',
+                '</div>',
+            ],
+        },
+        'include-start': {
+            node: null,
+            html: [
+                '<div class="CodeMirror-lintmarker" data-lint="include" data-fold="start">&nbsp;',
+                  '<svg viewBox="0 0 100 100">',
+                    '<polygon points="0,0 100,0 50,100" />',
+                  '</svg>',
+                '</div>',
+            ],
+        },
+        'include-end': {
+            node: null,
+            html: [
+                '<div class="CodeMirror-lintmarker" data-lint="include" data-fold="end">&nbsp;',
+                  '<svg viewBox="0 0 100 100">',
+                    '<polygon points="50,0 100,100 0,100" />',
+                  '</svg>',
+                '</div>',
+            ],
+        },
+    };
+
+    const markerFromTemplate = which => {
+        const template = markerTemplates[which];
+        if ( template.node === null ) {
+            const domParser = new DOMParser();
+            const doc = domParser.parseFromString(template.html.join(''), 'text/html');
+            template.node = document.adoptNode(qs$(doc, '.CodeMirror-lintmarker'));
+        }
+        return template.node.cloneNode(true);
+    };
+
+    const addMarker = (doc, lineHandle, marker, details) => {
+        if ( marker !== null && marker.dataset.lint !== details.value ) {
+            doc.setGutterMarker(lineHandle, 'CodeMirror-lintgutter', null);
+            if ( marker.dataset.lint === 'error' ) {
+                errorCount -= 1;
+            }
+            marker = null;
+        }
+        if ( marker === null ) {
+            marker = markerFromTemplate(details.value);
+            doc.setGutterMarker(lineHandle, 'CodeMirror-lintgutter', marker);
+            if ( details.value === 'error' ) {
+                errorCount += 1;
+            }
+        }
+        const msgElem = qs$(marker, '.msg');
+        if ( msgElem === null ) { return; }
+        msgElem.textContent = details.msg || '';
+    };
+
+    const removeMarker = (doc, lineHandle, marker) => {
+        doc.setGutterMarker(lineHandle, 'CodeMirror-lintgutter', null);
+        if ( marker.dataset.lint === 'error' ) {
+            errorCount -= 1;
+        }
+    };
+
+    const processDeletion = (doc, change) => {
+        let { from, to } = change;
+        doc.eachLine(from.line, to.line, lineHandle => {
+            const marker = extractMarker(lineHandle);
+            if ( marker === null ) { return; }
+            if ( marker.dataset.lint === 'error' ) {
+                errorCount -= 1;
+                marker.dataset.lint = 'void';
+            }
+        });
+    };
+
+    const processInsertion = (doc, deadline, change) => {
+        let { from, to } = change;
+        doc.eachLine(from, to, lineHandle => {
+            astParser.parse(lineHandle.text);
+            const markerDetails = extractMarkerDetails(doc, lineHandle);
+            const marker = extractMarker(lineHandle);
+            if ( markerDetails === undefined && marker !== null ) {
+                removeMarker(doc, lineHandle, marker);
+            } else if ( markerDetails !== undefined ) {
+                addMarker(doc, lineHandle, marker, markerDetails);
+            }
+            from += 1;
+            if ( (from & 0x0F) !== 0 ) { return; }
+            if ( deadline.timeRemaining() !== 0 ) { return; }
+            return true;
+        });
+        if ( from !== to ) {
+            return { from, to };
+        }
+    };
+
+    const processChangeset = (doc, deadline) => {
+        const cm = doc.getEditor();
+        cm.startOperation();
+        while ( changeset.length !== 0 ) {
+            const change = processInsertion(doc, deadline, changeset.shift());
+            if ( change === undefined ) { continue; }
+            changeset.unshift(change);
+            break;
+        }
+        cm.endOperation();
+        if ( changeset.length !== 0 ) {
+            return processChangesetAsync(doc);
+        }
+        includeset.clear();
+        CodeMirror.signal(doc.getEditor(), 'linterDone', { errorCount });
+    };
+
+    const processChangesetAsync = doc => {
+        if ( changesetTimer !== undefined ) { return; }
+        changesetTimer = self.requestIdleCallback(deadline => {
+            changesetTimer = undefined;
+            processChangeset(doc, deadline);
+        });
+    };
+
+    const onChanges = (cm, changes) => {
+        const doc = cm.getDoc();
+        for ( const change of changes ) {
+            const from = change.from.line;
+            const to = from + change.text.length;
+            changeset.push({ from, to });
+            processChangesetAsync(doc);
+        }
+    };
+
+    const onBeforeChanges = (cm, change) => {
+        const doc = cm.getDoc();
+        processDeletion(doc, change);
+    };
+
+    const foldRangeFinder = (cm, from) => {
+        const lineNo = from.line;
+        const lineHandle = cm.getDoc().getLineHandle(lineNo);
+        const marker = extractMarker(lineHandle);
+        if ( marker === null ) { return; }
+        if ( marker.dataset.fold === undefined ) { return; }
+        const foldName = marker.dataset.lint;
+        from.ch = lineHandle.text.length;
+        const to = { line: 0, ch: 0 };
+        const doc = cm.getDoc();
+        let depth = 0;
+        doc.eachLine(from.line, doc.lineCount(), lineHandle => {
+            const marker = extractMarker(lineHandle);
+            if ( marker === null ) { return; }
+            if ( marker.dataset.lint === foldName && marker.dataset.fold === 'start' ) {
+                depth += 1;
+                return;
+            }
+            if ( marker.dataset.lint !== foldName ) { return; }
+            if ( marker.dataset.fold !== 'end' ) { return; }
+            depth -= 1;
+            if ( depth !== 0 ) { return; }
+            to.line = lineHandle.lineNo();
+            return true;
+        });
+        return { from, to };
+    };
+
+    const onGutterClick = (cm, lineNo, gutterId, ev) => {
+        if ( ev.button !== 0 ) { return; }
+        if ( gutterId !== 'CodeMirror-lintgutter' ) { return; }
+        const doc = cm.getDoc();
+        const lineHandle = doc.getLineHandle(lineNo);
+        const marker = extractMarker(lineHandle);
+        if ( marker === null ) { return; }
+        if ( marker.dataset.fold === 'start' ) {
+            if ( ev.ctrlKey ) {
+                if ( dom.cl.has(marker, 'folded') ) {
+                    CodeMirror.commands.unfoldAll(cm);
+                } else {
+                    CodeMirror.commands.foldAll(cm);
+                }
+                doc.setCursor(lineNo);
+                return;
+            }
+            cm.foldCode(lineNo, {
+                widget: '\u00A0\u22EF\u00A0',
+                rangeFinder: foldRangeFinder,
+            });
+            return;
+        }
+        if ( marker.dataset.fold === 'end' ) {
+            let depth = 1;
+            let lineNo = lineHandle.lineNo();
+            while ( lineNo-- ) {
+                const prevLineHandle = doc.getLineHandle(lineNo);
+                const markerFrom = extractMarker(prevLineHandle);
+                if ( markerFrom === null ) { continue; }
+                if ( markerFrom.dataset.fold === 'end' ) {
+                    depth += 1;
+                } else if ( markerFrom.dataset.fold === 'start' ) {
+                    depth -= 1;
+                    if ( depth === 0 ) {
+                        doc.setCursor(lineNo);
+                        break;
+                    }
+                }
+            }
+            return;
+        }
+    };
+
+    self.addEventListener('filterOnHeaders', ( ) => {
+        astParser.options.filterOnHeaders = filterOnHeaders;
+    });
+
+    CodeMirror.defineInitHook(cm => {
+        cm.on('changes', onChanges);
+        cm.on('beforeChange', onBeforeChanges);
+        cm.on('gutterClick', onGutterClick);
+        cm.on('fold', function(cm, from) {
+            const doc = cm.getDoc();
+            const lineHandle = doc.getLineHandle(from.line);
+            const marker = extractMarker(lineHandle);
+            dom.cl.add(marker, 'folded');
+        });
+        cm.on('unfold', function(cm, from) {
+            const doc = cm.getDoc();
+            const lineHandle = doc.getLineHandle(from.line);
+            const marker = extractMarker(lineHandle);
+            dom.cl.remove(marker, 'folded');
+        });
+    });
+}
+
+/******************************************************************************/
+
 // Enhanced word selection
 
 {
