@@ -368,6 +368,14 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
 
 /******************************************************************************/
 
+µb.isTrustedList = function(assetKey) {
+    if ( assetKey.startsWith('ublock-') ) { return true; }
+    if ( assetKey === this.userFiltersPath ) { return true; }
+    return false;
+};
+
+/******************************************************************************/
+
 µb.loadSelectedFilterLists = async function() {
     const bin = await vAPI.storage.get('selectedFilterLists');
     if ( bin instanceof Object && Array.isArray(bin.selectedFilterLists) ) {
@@ -560,7 +568,8 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
     await this.saveUserFilters(details.content.trim() + '\n' + filters);
 
     const compiledFilters = this.compileFilters(filters, {
-        assetKey: this.userFiltersPath
+        assetKey: this.userFiltersPath,
+        isTrusted: true,
     });
     const snfe = staticNetFilteringEngine;
     const cfe = cosmeticFilteringEngine;
@@ -679,7 +688,9 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
     // Load previously saved available lists -- these contains data
     // computed at run-time, we will reuse this data if possible
     const [ bin, registeredAssets, badlists ] = await Promise.all([
-        vAPI.storage.get('availableFilterLists'),
+        Object.keys(this.availableFilterLists).length !== 0
+            ? { availableFilterLists: this.availableFilterLists }
+            : vAPI.storage.get('availableFilterLists'),
         io.metadata(),
         this.badLists.size === 0 ? io.get('ublock-badlists') : false,
     ]);
@@ -883,10 +894,10 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
         if ( µb.inMemoryFilters.length !== 0 ) {
             if ( µb.inMemoryFiltersCompiled === '' ) {
                 µb.inMemoryFiltersCompiled =
-                    µb.compileFilters(
-                        µb.inMemoryFilters.join('\n'),
-                        { assetKey: 'in-memory'}
-                    );
+                    µb.compileFilters(µb.inMemoryFilters.join('\n'), {
+                        assetKey: 'in-memory',
+                        isTrusted: true,
+                    });
             }
             if ( µb.inMemoryFiltersCompiled !== '' ) {
                 toLoad.push(
@@ -951,8 +962,10 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
         return { assetKey, content: '' };
     }
 
-    const compiledContent =
-        this.compileFilters(rawDetails.content, { assetKey });
+    const compiledContent = this.compileFilters(rawDetails.content, {
+        assetKey,
+        isTrusted: this.isTrustedList(assetKey),
+    });
     io.put(compiledPath, compiledContent);
 
     return { assetKey, content: compiledContent };
@@ -1018,6 +1031,7 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
     // client compilers.
     if ( details.assetKey ) {
         writer.properties.set('name', details.assetKey);
+        writer.properties.set('isTrusted', details.isTrusted === true);
     }
     const assetName = details.assetKey ? details.assetKey : '?';
     const expertMode =
@@ -1469,18 +1483,12 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
         // Respect cooldown period before launching an emergency update.
         const timeSinceLastEmergencyUpdate = (now - lastEmergencyUpdate) / 3600000;
         if ( timeSinceLastEmergencyUpdate > 1 ) {
-            const assetDict = await io.metadata();
-            for ( const [ assetKey, asset ] of Object.entries(assetDict) ) {
-                if ( asset.hasRemoteURL !== true ) { continue; }
-                if ( asset.content === 'filters' ) {
-                    if ( µb.selectedFilterLists.includes(assetKey) === false ) {
-                        continue;
-                    }
-                }
-                if ( asset.obsolete !== true ) { continue; }
-                const lastUpdateInDays = (now - asset.writeTime) / 86400000;
-                const daysSinceVeryObsolete = lastUpdateInDays - 2 * asset.updateAfter;
-                if ( daysSinceVeryObsolete < 0 ) { continue; }
+            const entries = await io.getUpdateAges({
+                filters: µb.selectedFilterLists,
+                internal: [ '*' ],
+            });
+            for ( const entry of entries ) {
+                if ( entry.ageNormalized < 2 ) { continue; }
                 needEmergencyUpdate = true;
                 lastEmergencyUpdate = now;
                 break;
@@ -1543,7 +1551,8 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
                         io.put(
                             'compiled/' + details.assetKey,
                             this.compileFilters(details.content, {
-                                assetKey: details.assetKey
+                                assetKey: details.assetKey,
+                                isTrusted: this.isTrustedList(details.assetKey),
                             })
                         );
                     }
@@ -1629,7 +1638,11 @@ self.addEventListener('hiddenSettingsChanged', ( ) => {
         if ( newDefaultListset.size === 0 ) { return; }
         if ( oldDefaultListset.size === 0 ) {
             Array.from(Object.entries(oldDict))
-                .filter(a => a[1].content === 'filters' && a[1].off === undefined)
+                .filter(a =>
+                    a[1].content === 'filters' &&
+                    a[1].off === undefined &&
+                    /^https?:\/\//.test(a[0]) === false
+                )
                 .map(a => a[0])
                 .forEach(a => oldDefaultListset.add(a));
             if ( oldDefaultListset.size === 0 ) { return; }
